#include "mixedGraph.h"
#include "evenDegree.h"
#include "inOutDegree.h"
#include "evenParity.h"


static void getTotalCost(MixedGraph<NodeInfo,EdgeInfo>& graph,
			   int& total_cost, int& narcs, int& nedges);


int EdmondsAndJohnson(MixedGraph<NodeInfo,EdgeInfo>& graph,
            int& total_cost, int& narcs, int& nedges) {

   GRAPH<node,int> EC;

   
   #ifdef PRINTLEVEL
      cout << endl <<"Calling algorithm EVENDEGREE:" << endl;
   #endif
   if (evenDegree(graph)) {
      cerr << "Algorithm EVENDEGREE failed!" << endl;
      return 1;
   }


   #ifdef PRINTLEVEL
      cout << endl << "Calling algorithm INOUTDEGREE:" << endl;
   #endif
   if (inOutDegree(graph,EC)) {
      cerr << "Algorithm INOUTDEGREE failed!" << endl;
      return 1;
   }


   #ifdef PRINTLEVEL
      cout << endl << "Calling algorithm EVENPARITY:" << endl;
   #endif
   if (evenParity(graph,EC)) {
      cerr << "Algorithm EVENPARITY failed!" << endl;
      return 1;
   }
   
   getTotalCost(graph,total_cost,narcs,nedges);

   return 0;
}        



static void getTotalCost(MixedGraph<NodeInfo,EdgeInfo>& graph,
                 int& total_cost, int& narcs, int& nedges) {

   total_cost = 0;
   narcs = 0;
   nedges = 0;

   edge e;
 
   forall_edges(e,graph.getDiGraph()) {
      total_cost += graph[e].multi * graph[e].cost; 
      narcs += graph[e].multi;
   }
       
   forall_edges(e,graph.getUGraph()) { 
      total_cost += graph[e].multi * graph[e].cost;    
      nedges += graph[e].multi;
   }
 
}
   
