/** \file edmondsAndJohnsonMCPP.h
 * Exported function: EdmondsAndJohnson().
 */

#ifndef EDMONDSANDJOHNSONMCPP_H
#define EDMONDSANDJOHNSONMCPP_H

#include "mixedGraph.h"


/** Performs the Edmonds and Johnson Mixed Postman Algorithm.
 *  This function takes as argument a connected mixed graph, adds
 *  some arcs/edges and orient some edges. After its termination
 *  it will satisfy the following property: 
 *  The degree of each node is even and indegree equals outdegree
 *  in each node.
 *  \param graph A reference to a connected mixed graph graph of type 
 *  MixedGraph<NodeInfo,EdgeInfo>.
 *  \param total_cost A reference to an int. It will return the 
 *  total cost of all arcs/edges of graph at the end of the algorithm.
 *  \param narcs A reference to an int. It will return the number
 *  of all arcs of graph at the end of the algorithm. Note, that this
 *  information cannot be accessed through graph.numberOfArcs(), because
 *  this ignores the "multi"-info of the arcs.
 *  \param nedges A reference to an int. It will return the number
 *  of all edges of graph at the end of the algorithm. Note, that this
 *  information cannot be accessed through graph.numberOfEdges(), because
 *  this ignores the "multi"-info of the edges.
 *  \return 0, if it worked
 *  \return 1, otherwise 
 */  
int EdmondsAndJohnson(MixedGraph<NodeInfo,EdgeInfo>& graph, int&,int&,int&);

#endif

