#include "mixedGraph.h"


static int adjustcycles(MixedGraph<NodeInfo,EdgeInfo>&, 
                        GRAPH<node,int>& extraCopies, list<node>&);


 
int evenParity(MixedGraph<NodeInfo,EdgeInfo>& graph, GRAPH<node,int>& extraCopies) {

   /* Identify the odd nodes of graph and put them into the list "oddNodes". 
   */

   #ifdef PRINTLEVEL
      cout << "  Identifying the odd nodes..." << endl;
   #endif
   
   list<node> oddNodes;
   int count = 0;
   node n;
   forall_nodes(n,graph.getDiGraph()) {
      if (graph[n].degree%2 == 1) {
         count++;
         oddNodes.append(n);
      }
   }

   #ifdef PRINTLEVEL
      cout << "  ...found " << count << " odd nodes." << endl;
   #endif

   
   
   /* If there are some odd nodes, call algorithm ADJUSTCYCLES.
   */
  
   #ifdef PRINTLEVEL
      if (count)
         cout << "  Calling algorithm ADJUSTCYCLES:" << endl;
   #endif
 
   if (count)
      adjustcycles(graph,extraCopies,oddNodes);   
 
   return 0;
}   




static int adjustcycles(MixedGraph<NodeInfo,EdgeInfo>& graph, GRAPH<node,int>& extraCopies, list<node>& oddNodes) {

   /* Create a node_array<node> extraCopiesNodes such that every node v
      of graph corresponds to the node extraCopiesNodes[v] of extraCopies.
   */

   node_array<node> extraCopiesNodes(graph.getDiGraph());
   NodeIt diGraphIt(graph.getDiGraph());
   NodeIt extraCopiesIt(extraCopies);
 
   for (; diGraphIt.valid(); diGraphIt++,extraCopiesIt++) 
      extraCopiesNodes[diGraphIt.get_node()] = extraCopiesIt.get_node(); 


   /* ***************** Algorithm: ****************** */ 

   /* In the following, let V be the set of odd nodes of graph.
      The variables v and w will represent nodes of graph.
      
      Note that the undirected subgraph uGraph of graph contains all
      those edges of the initial graph, that have not been oriented by 
      algorithm inOutDegree.
   */

   while (!oddNodes.empty()) {   // "While V is not empty"
 
      node v = oddNodes.front();   // "Select v from V
      node vs = v;                  //  and set vs to v"

      list_item listItem;
      while ( (listItem=oddNodes.search(v)) != nil ) {
	// "While v is in V"
 
	 oddNodes.del_item(listItem); // "Remove v from V"
 
         do {

       	    // "Choose an edge [extraCopiesNodes[v],extraCopiesNodes[w]]
	    // from extraCopies:" 

            edge e = extraCopies.first_adj_edge(extraCopiesNodes[v]);
            EdgeInfo inf;
            node w;  

            if (e != nil) {
            // There is an arc <extraCopiesNodes[v],extraCopiesNodes[w]>
            // in extraCopies.
 

               w = extraCopies[extraCopies.target(e)]; 
               
               // "Insert a duplicate copy of <v,w> into graph":
               inf.multi = 1; 
               graph.insertArc(v,w,inf);
               

               #ifdef PRINTLEVEL
                  cout << "    duplicate";
                  graph.printNode(v); cout << "-->"; graph.printNode(w);
                  cout << endl;
               #endif

            }

            else {
            // There must be an arc <extraCopiesNodes[w],extraCopiesNodes[v]>
	    // in extraCopies.

               e = extraCopies.first_in_edge(extraCopiesNodes[v]);
               w = extraCopies[extraCopies.source(e)];

               // "Delete a copy of <w,v> from graph"
               inf.multi = -1;
               graph.insertArc(w,v,inf);
              

               #ifdef PRINTLEVEL
                  cout << "    delete";
                  graph.printNode(w); cout << "-->"; graph.printNode(v);
                  cout << endl;
               #endif

            }

            
            // Remove one copy of [extraCopiesNodes[v],extraCopiesNodes[w]]
	    // from extraCopies:

            int extraCopiesMulti = extraCopies.inf(e)-1;

            if (extraCopiesMulti)  // There is at least one copy of e left.
               extraCopies.assign(e,extraCopiesMulti);
            else  // The last copy of e has been removed.
               extraCopies.del_edge(e);


         
            v = w;

         } while ( (listItem = oddNodes.search(v)) == nil);
         //  "while v is not in V"
         
         oddNodes.del_item(listItem);  // "Remove v from V"
           
         do {
                        
	    // "Choose (v,w) from graph:

            edge e = graph.firstAdjEdge(v);
            EdgeInfo inf = graph[e];
            node w = (graph.source(e)==v) ? graph.target(e) : graph.source(e);

            #ifdef PRINTLEVEL
               cout << "    orient"; 
               graph.printNode(v); cout << "-->"; graph.printNode(w);
               cout << endl;
            #endif


	    // "Orient (v,w) from v to w and insert it into graph:"
            inf.multi = 1;
            graph.insertArc(v,w,inf);

            // "Remove one copy of (v,w) from graph"
            inf.multi = -1;
            graph.insertEdge(v,w,inf);

            if (!graph[e].multi)  // last occurence of (v,w) deleted... 
	       graph.delEdge(e); // ...delete edge completely  

           
            v = w;
         } while (oddNodes.search(v) == nil && v != vs);
	 // "while v is not in V and v!=vs"
      }
      
   }
   
   return (0);
}
    
