/** \file inOutDegree.h
 * Exported function: inOutDegree().
*/
#ifndef INOUTDEGREE_H
#define INOUTDEGREE_H

#include "mixedGraph.h"


/** Makes indegree equal to outdegree in each node.
 * This function takes a connected mixed graph and makes indegree
 * equal to outdegree in each node by
 *   - orienting some edges and/or 
 *   - duplicating some oriented edges and/or
 *   - duplicating some arcs,
 * while minimizing the cost of the added arcs/oriented edges.
 * \param graph A reference to a mixed graph of type MixedGraph<NodeInfo,EdgeInfo>.
 * \param extraCopies A reference to an (empty) directed graph of type GRAPH<node,int>.
 *
 * Afterwards, for this graph will hold the following: 
 *   - extraCopies has graph.numberOfNodes() nodes.
 *   - Each node v of extraCopies corresponds to the node extraCopies[v] of graph.
 *   - For each arc <v,w> of extraCopies (with integer arc information \a inf): 
 *     The algorithm has made \a inf additional copies of the arc or 
 *     oriented edge <extraCopies[v],extraCopies[w]> of graph.
 * This graph will be needed in algorithm ::evenParity.
 * \return 0, if it worked.
 * \return 1, otherwise.
 */
int inOutDegree(MixedGraph<NodeInfo,EdgeInfo>& graph, GRAPH<node,int>& extraCopies);



#endif


