#include <LEDA/string.h>
#include <LEDA/array.h>
#include <fstream.h>
#include <LEDA/stream.h> 
#include <LEDA/error.h>

#include "mixedGraph.h"
#include "edmondsAndJohnsonMCPP.h"


int main(int argc, char *argv[]) {
 
   ifstream Source;  

   if (argc == 1) {
     cout << "Program call:" << endl;
     cout << "   HEURISTIK file" << endl 
       << "where file contains a mixed graph given in the following format:";
     cout << endl << endl;
     cout << "(dummy string) (dummy integer) (dummy float)" << endl   
          << "(number of nodes) (number of nodes)" << endl
          << "(number of edges) (number of edges)" << endl
          << "(number of arcs) (number of arcs)" << endl
          << "(source node) (target node) (cost)" << endl
          << "(source node) (target node) (cost)" << endl
          << "    ...            ...        ... " << endl
          << endl << "all without the parantheses." << endl;
     return 0;
   }
 
   string Name = argv[1];
   Source.open(Name, ios::in);
    
   if (!Source)
     cout << "Fehler";

   catch_system_errors(); 


   string expr;    
   Source >> expr; Source >> expr;
       
   float lower;
   Source >> lower;

   int nOfNodes;
   Source >> nOfNodes; Source >> nOfNodes;

   int nOfEdges;
   Source >> nOfEdges; Source >> nOfEdges;

   int nOfArcs;
   Source >> nOfArcs; Source >> nOfArcs;

   
   NodeInfo startInfo;
   startInfo.degree = startInfo.indegree = startInfo.outdegree = 0;

   MixedGraph<NodeInfo,EdgeInfo> graph(nOfNodes,startInfo);

   array<node> nodes(1,nOfNodes);
   NodeIt nIt(graph.getDiGraph());
   int count = 1;
   for (; nIt.valid(); nIt++,count++)
     nodes[count] = nIt.get_node();


   for (int i=0; i<nOfEdges; i++) {
     int intNode1;
     int intNode2;
     int cost;
   
     Source >> intNode1; 
     Source >> intNode2; 
     Source >> cost;
     
     EdgeInfo info;
     info.multi = 1; 
     info.cost = cost;

     graph.insertEdge(nodes[intNode1],nodes[intNode2],info);
   }
 
   
   for (int i=0; i<nOfArcs; i++) {
     int intSource;
     int intTarget;
     int cost;

     Source >> intSource; 
     Source >> intTarget;
     Source >> cost;
     
     EdgeInfo info;
     info.multi = 1;
     info.cost = cost;

     graph.insertArc(nodes[intSource],nodes[intTarget],info);   
   }


   cout << endl << "Problem: " << Name << endl << endl;
   cout << "Initial graph:" << endl;
   graph.print();


   int totalCost;
   int newNOfArcs;
   int newNOfEdges;

   cout << endl << "Performing the Edmonds and Johnson Mixed Chinese \
Postman Algorithm:" << endl;
   EdmondsAndJohnson(graph,totalCost,newNOfArcs,newNOfEdges);

   cout << endl << "Final graph:" << endl;
   graph.print();

   cout << endl << "Number of additional copies of arcs/edges made by"
	  << endl << " algorithm EDMONDS_AND_JOHNSON:"
	  << (newNOfArcs+newNOfEdges) - (nOfArcs+nOfEdges) << endl;

   cout << "Total cost of the tour found by algorithm " << 
       "edmondsAndJohnson: " << totalCost << endl;
 
   cout << "Comparison value: " << lower << endl; 
    
   return 0;
}
