

/* *************************************************************** */
/*      Definitions of the member functions of class MixedGraph    */ 
/* *************************************************************** */


template <class VType, class EType>
inline MixedGraph<VType,EType>::MixedGraph(int n) 
  : nodeCount(n), uNodes(diGraph,n,nil), diNodes(uGraph,n,nil) {

   for (int i=1; i<=n; i++) { 
      node newDiNode = diGraph.new_node();
      node newUNode = uGraph.new_node();

      uNodes[newDiNode] = newUNode;
      diNodes[newUNode] = newDiNode;
   }
}   


template <class VType, class EType>
inline MixedGraph<VType,EType>::MixedGraph(int n, VType startInfo) 
  : nodeCount(n), uNodes(diGraph,n,nil), diNodes(uGraph,n,nil) {

   for (int i=1; i<=n; i++) { 
      node newDiNode = diGraph.new_node(startInfo);
      node newUNode = uGraph.new_node(startInfo);

      uNodes[newDiNode] = newUNode;
      diNodes[newUNode] = newDiNode;
   }
}



template <class VType, class EType>
inline node MixedGraph<VType,EType>::source(edge e) const {
  return(isArc(e) ? diGraph.source(e) : diNodes[uGraph.source(e)]);
}


template <class VType, class EType>
inline node MixedGraph<VType,EType>::target(edge e) const {
  return isArc(e) ? diGraph.target(e) : diNodes[uGraph.target(e)];
}


template <class VType, class EType>
inline int MixedGraph<VType,EType>::numberOfNodes() const {
   return nodeCount;
}


template <class VType, class EType>
inline int MixedGraph<VType,EType>::numberOfArcs() const {
   return diGraph.number_of_edges();
}


template <class VType, class EType>
inline int MixedGraph<VType,EType>::numberOfEdges() const {
   return uGraph.number_of_edges();
}


template <class VType, class EType>
inline list<node> MixedGraph<VType,EType>::allNodes() const {
   return diGraph.all_nodes();
}


template <class VType, class EType>
inline list<edge> MixedGraph<VType,EType>::allArcs() const {
   return diGraph.all_edges();
}


template <class VType, class EType>
inline list<edge> MixedGraph<VType,EType>::allEdges() const {
   return uGraph.all_edges();
}



template <class VType, class EType>
inline node MixedGraph<VType,EType>::firstNode() const {
   return diGraph.first_node();
}


template <class VType, class EType>
inline node MixedGraph<VType,EType>::succNode(node v) const {
   return diGraph.succ_node(v);
}


template <class VType, class EType>
inline node MixedGraph<VType,EType>::predNode(node v) const {
   return diGraph.pred_node(v);
}


template <class VType, class EType>
inline node MixedGraph<VType,EType>::lastNode() const {
   return diGraph.last_node();
}



template <class VType, class EType>
inline edge MixedGraph<VType,EType>::firstArc() const {
   return diGraph.first_edge();
}


template <class VType, class EType>
inline edge MixedGraph<VType,EType>::succArc(edge a) const {
   return diGraph.succ_edge(a);
}


template <class VType, class EType>
inline edge MixedGraph<VType,EType>::predArc(edge a) const {
   return diGraph.pred_edge(a);
}


template <class VType, class EType>
inline edge MixedGraph<VType,EType>::lastArc() const {
   return diGraph.last_edge();
}


template <class VType, class EType>
inline edge MixedGraph<VType,EType>::firstEdge() const {
   return uGraph.first_edge();
}


template <class VType, class EType>
inline edge MixedGraph<VType,EType>::succEdge(edge e) const {
   return uGraph.succ_edge(e);
}


template <class VType, class EType>
inline edge MixedGraph<VType,EType>::predEdge(edge e) const {
   return uGraph.pred_edge(e);
}


template <class VType, class EType>
inline edge MixedGraph<VType,EType>::lastEdge() const {
   return uGraph.last_edge();
}


template <class VType, class EType>
inline edge MixedGraph<VType,EType>::firstAdjArc(node v) const {
   return diGraph.first_adj_edge(v);
}


template <class VType, class EType>
inline edge MixedGraph<VType,EType>::firstInArc(node v) const {
   return diGraph.first_in_edge(v);
}


template <class VType, class EType>
inline edge MixedGraph<VType,EType>::firstAdjEdge(node v) const {
   return uGraph.first_adj_edge(uNodes[v]);
}


template <class VType, class EType>
inline edge MixedGraph<VType,EType>::adjSucc(edge e) const {
   return isArc(e) ? diGraph.adj.succ(e) : uGraph.adj_succ(e);
}



template <class VType,class EType>
edge MixedGraph<VType,EType>::findArc(node source, node target) const {

   // No selfloops !
   if (source == target)
      return nil;

   /* Iterate over the arcs ending in target and over the arcs starting
      in source until either arc <source,target> is found or there are
      no arcs left to check.
   */

   InAdjIt inIt(diGraph,target);
   OutAdjIt outIt(diGraph,source);
 
   bool found1 = false;
   bool found2 = false;

   while (1) {

      found1 = inIt.valid() && diGraph.source(inIt.get_edge()) == source;
      found2 = outIt.valid() && diGraph.target(outIt.get_edge()) == target;
      bool notFound = !inIt.valid() && !outIt.valid();
 
      if (found1 || found2 || notFound)
         break;

      if (inIt.valid()) ++inIt;
      if (outIt.valid()) ++outIt;
   }
   
   // If the arc is found, then return it.

   if (found1)
      return(inIt.get_edge());
   if (found2)
      return(outIt.get_edge());

   // arc not found   
   return nil;
} 


template <class VType, class EType>
inline edge MixedGraph<VType,EType>::insertArc(node source, node target) {
   return diGraph.new_edge(source,target);
}

template <class VType, class EType>
inline edge MixedGraph<VType,EType>::insertArc(node source, node target, EType newArcInfo) {
   return diGraph.new_edge(source,target,newArcInfo);
}   



/* A specialized version of insert_arc for the case that 
   <VType,EType>==<NodeInfo,EdgeInfo>
*/
inline edge MixedGraph<NodeInfo,EdgeInfo>::insertArc(node source, node target, EdgeInfo newArcInfo) {

   int multi = newArcInfo.multi;
   bool found = false;
   edge e;
  
   /* If arc <source,target> already exists, only the related arc and node
      data must be updated. Otherwise, the arc must be created.
   */
   found = ( (e=findArc(source,target)) != nil);
   if (!found)
      e = diGraph.new_edge(source,target,newArcInfo);


   NodeInfo sourceInfo = diGraph[source];
   NodeInfo targetInfo = diGraph[target];

   // Updating the degree of source and target:
   sourceInfo.degree += multi;
   targetInfo.degree += multi;
   sourceInfo.outdegree += multi;
   targetInfo.indegree += multi;
   
   diGraph.assign(source,sourceInfo);
   diGraph.assign(target,targetInfo);
   
   if (found) {  // Updating the multiplicity of the arc.
      EdgeInfo info = diGraph[e];
      info.multi += multi;
      diGraph.assign(e,info);
   }  
      
   return e;
}


template <class VType,class EType>
void MixedGraph<VType,EType>::delArc(edge a) {
   diGraph.del_edge(a);
}



template <class VType,class EType>
edge MixedGraph<VType,EType>::findEdge(node node1, node node2) const {
 
   // No selfloops!
   if (node1 == node2)
     return nil;
   

   /* node1,node2 are nodes of diGraph. Translate them into the nodes v,w
      of uGraph.
   */ 
   node v = uNodes[node1];  
   node w = uNodes[node2];


   /* Iterate over the edges adjacent to v and over the edges adjacent 
      to w until either edge (v,w) is found or there are
      no edges left to check.
   */

   list<edge> adj1 = uGraph.adj_edges(v); // Adjacency list of v.
   list<edge> adj2 = uGraph.adj_edges(w); // Adjacency list of w.

   edge e1=nil;
   edge e2=nil;     
   bool found1 = false;
   bool found2 = false;

   while (1) {  

      if (!adj1.empty())
         e1 = adj1.pop();  // next element in the adjacency list of v 
      else
         e1 = nil;  // all edges adjacent to v already checked

      if (!adj2.empty())
         e2 = adj2.pop();  // next element in the adjacency list of w
      else
         e2 = nil;  // all edges adjacent to w already checked

      found1 = (e1!=nil && (uGraph.source(e1)==w || uGraph.target(e1)==w));
      found2 = (e2!=nil && (uGraph.source(e2)==v || uGraph.target(e2)==v));
      bool notFound = adj1.empty() && adj2.empty();        

      if (found1 || found2 || notFound)
         break;

   } 

   
   // If the edge (v,w) is found, then return it.

   if (found1) 
      return(e1);
   if (found2) 
      return(e2);
   
   // edge not found
   return nil;
} 



template <class VType, class EType>
inline edge MixedGraph<VType,EType>::insertEdge(node node1, node node2) {
   return uGraph.new_edge(uNodes[node1],uNodes[node2]);
}



template <class VType, class EType>
inline edge MixedGraph<VType,EType>::insertEdge(node node1, node node2, EType newEdgeInfo) {
   return uGraph.new_edge(uNodes[node1],uNodes[node2],newEdgeInfo);
}



/* A specialized version of insert_edge for the case that
   <VType,EType>==<NodeInfo,EdgeInfo>.
*/

inline edge MixedGraph<NodeInfo,EdgeInfo>::insertEdge(node node1, node node2, EdgeInfo newEdgeInfo) {
   const int multi = newEdgeInfo.multi;

   node v = uNodes[node1];   // diGraph-nodes-->uGraph-nodes
   node w = uNodes[node2];   // --------"---------


   /* If edge (v,w) already exists, only the related edge and node 
      data must be updated. Otherwise, the edge must be created.
   */
 
   edge e;
   bool found = ( (e=findEdge(node1,node2)) != nil);
   if (!found)
      e = uGraph.new_edge(v,w,newEdgeInfo);


   NodeInfo sourceInfo = diGraph[node1];
   NodeInfo targetInfo = diGraph[node2];

   // Updating the degree of source and target.
   sourceInfo.degree += multi;
   targetInfo.degree += multi; 

   diGraph.assign(node1,sourceInfo);
   diGraph.assign(node2,targetInfo);
      

   if (found) {  // Updating the multiplicity of the edge.
      EdgeInfo info = uGraph[e];
      info.multi += multi;
      uGraph.assign(e,info);
   }  

  
   return e;
}



template <class VType,class EType>
void MixedGraph<VType,EType>::delEdge(edge e) {
   uGraph.del_edge(e);
}



template <class VType, class EType>
inline void MixedGraph<VType,EType>::printNode(node n, ostream& O) const {
   diGraph.print_node(n,O);
}


template <class VType, class EType>
inline void MixedGraph<VType,EType>::printArc(edge a, ostream& O) const {
   diGraph.print_edge(a,O);
}


template <class VType, class EType>
inline void MixedGraph<VType,EType>::printEdge(edge e, ostream& O) const {
   uGraph.print_edge(e,O);
}   



template <class VType, class EType>
inline void MixedGraph<VType,EType>::print(ostream& O) const { 
   node v;
   forall_nodes(v,diGraph) {
      O << endl;
      diGraph.print_node(v,O);
      O << ": ";

      edge e;
      forall_out_edges(e,v)
         diGraph.print_edge(e,O);
      forall_adj_edges(e,uNodes[v])
         uGraph.print_edge(e,O);

   }
   O << endl;
   
}
            


/* A specialized version of G.print for the case that 
   <VType,EType>==<NodeInfo,EdgeInfo>.
   Also the total number of arcs and edges (considering e.multi
   for all arcs/edges e !) and the total cost of all arcs/edges
   are printed on the output stream.
*/   
inline void MixedGraph<NodeInfo,EdgeInfo>::print(ostream& O) const {
   int edgeCost = 0; 
   int totalCost = 0;
   int arcCount = 0;   
   int edgeCount = 0;

   node v;
   forall_nodes(v,diGraph) {
      O << endl;
      diGraph.print_node(v,O);
      O << ": ";

      edge e;

      forall_out_edges(e,v) {
         diGraph.print_edge(e,O);
         
         arcCount += (*this)[e].multi;
         totalCost += (*this)[e].multi * (*this)[e].cost; 
      }

      forall_adj_edges(e,uNodes[v]) {
         uGraph.print_edge(e,O);
 
         edgeCount += (*this)[e].multi;
         edgeCost += (*this)[e].multi * (*this)[e].cost;    
      }
      
   }
   
   edgeCount /= 2; // Each edge appears twice in the adj. lists !
   totalCost += edgeCost/2;
 
   

   O << endl << "Total cost: " << totalCost;
   O << "   Number of arcs: " << arcCount;
   O << "   Number of edges: " << edgeCount;
   O << endl;
   
}
       


template <class VType, class EType>
inline VType MixedGraph<VType,EType>::inf(node v) const {
   return diGraph[v];
}


template <class VType, class EType>
inline const VType& MixedGraph<VType,EType>::operator[](node v) const {
   return diGraph[v];
}


template <class VType, class EType>
inline EType MixedGraph<VType,EType>::inf(edge e) const {
   return isArc(e) ? diGraph[e] : uGraph[e];
}

template <class VType, class EType>
inline const EType& MixedGraph<VType,EType>::operator[](edge e) const {
   return isArc(e) ? diGraph[e] : uGraph[e];
}


template <class VType, class EType>
inline void MixedGraph<VType,EType>::assign(node v, VType x) {
   diGraph.assign(v,x);
}


template <class VType, class EType>
inline void MixedGraph<VType,EType>::assign(edge e, EType x) {
   if (isArc(e))
      diGraph.assign(e,x);
   else 
      uGraph.assign(e,x);
}



/* In case that <VType,EType>==<NodeInfo,EdgeInfo>:
   "assign" also updates the (in/out)degree of the incident nodes,
   which should change by assigning a new arc/edge info.
*/ 
inline void MixedGraph<NodeInfo,EdgeInfo>::assign(edge e, EdgeInfo newInfo) {
 
   if (isArc(e)) {

      EdgeInfo oldInfo = diGraph[e];
      diGraph.assign(e,newInfo);


      // Updating the degree of source and target of the arc:

     const int deltaMulti = newInfo.multi - oldInfo.multi;
 
     NodeInfo sourceInfo = diGraph[diGraph.source(e)];
     NodeInfo targetInfo = diGraph[diGraph.target(e)];

     sourceInfo.degree += deltaMulti;
     targetInfo.degree += deltaMulti;
     targetInfo.indegree += deltaMulti;
     sourceInfo.outdegree += deltaMulti;
     
     diGraph.assign(diGraph.source(e),sourceInfo);
     diGraph.assign(diGraph.target(e),targetInfo);
  
   }
   else {

     EdgeInfo oldInfo = uGraph[e];
     uGraph.assign(e,newInfo);


     /* Updating the degree of the endpoints of the edge.
        Note that these updates must be made for the corresponding
        nodes of diGraph!
     */
     const int deltaMulti = newInfo.multi - oldInfo.multi;

     NodeInfo sourceInfo = diGraph[diNodes[uGraph.source(e)]];
     NodeInfo targetInfo = diGraph[diNodes[uGraph.target(e)]];

     sourceInfo.degree += deltaMulti;
     targetInfo.degree += deltaMulti;

     diGraph.assign(diNodes[uGraph.source(e)],sourceInfo);
     diGraph.assign(diNodes[uGraph.target(e)],targetInfo);  
   }
} 



template <class VType, class EType>
inline bool MixedGraph<VType,EType>::isNode(node v) const {
   return graph_of(v) == (leda_graph*)&diGraph;
}


template <class VType, class EType>
inline bool MixedGraph<VType,EType>::isArc(edge a) const {
   return graph_of(a) == (leda_graph*)&diGraph;
}


template <class VType, class EType>
inline bool MixedGraph<VType,EType>::isEdge(edge e) const {
   return graph_of(e)==(leda_graph*)&uGraph;
}


template <class VType, class EType>
inline const GRAPH<VType,EType>& MixedGraph<VType,EType>::getDiGraph() const {
   return diGraph;
}


template <class VType, class EType>
inline const UGRAPH<VType,EType>& MixedGraph<VType,EType>::getUGraph() const {
   return uGraph;
}































